/****************************************************************
* Modul:   TA.BA_MMT.HS1301 - Multimedia Technik (MMT)
* Subject: Processing - Uebung 4
* Author:  Felix Rohrer
* Desc:    MP3 Player
****************************************************************/
import java.io.*;
import java.util.*;
import java.util.concurrent.TimeUnit;
import ddf.minim.*;
import ddf.minim.analysis.*;


// ----- init Vars ----------------------------------------------
int canvasWidth = 450;
int canvasHeight = 220;
int currentSongIndex = 0;

Minim minim;
File folder;
List<AudioPlayer> songList = new ArrayList<AudioPlayer>();
boolean isPlaying = false;
AudioPlayer currentSong;
FFT fft;

// ----- setup() ------------------------------------------------
void setup() {
  size(canvasWidth, canvasHeight);
  frameRate(24);
  background(45);
  textSize(16);
  
  minim = new Minim(this);

  // get all files from data folder, create playlist
  folder = new File(dataPath(""));
  for (String file : folder.list()) {
    if (file.toLowerCase().endsWith(".mp3")) {
      songList.add(minim.loadFile(file));
    }
  }
  
  currentSong = songList.get(currentSongIndex);
  // init FFT
  fft = new FFT(currentSong.bufferSize(), currentSong.sampleRate());
  fft.window(FFT.HAMMING);
  
  currentSong.play();
  isPlaying = true;
}

// ----- draw() -------------------------------------------------
void draw() {
  background(45);
  
  // Play Soundlist
  playSound();
  
  // draw waveform
  stroke(255);
  saturation(10);
  for(int i = 0; i < currentSong.bufferSize()-1; i++)
  {
    line(i,   80 + currentSong.mix.get(i)*50,
         i+1, 80 + currentSong.mix.get(i+1)*50);
  }
  
  // draw FFT
  stroke(175);
  fft.forward(currentSong.mix);
  for (int i = 0; i < fft.specSize(); i++) {
    line(i, height, i, height - fft.getBand(i) * 4);
  }
 
  // fetch current play position
  float playPos = currentSong.position();
  float playLen = currentSong.length();
  float xpos = (playPos / playLen) * width;
  
  // draw progress bar
  fill(alpha(color(0,126,255,102)));
  stroke(alpha(color(0,126,255,102)));
  rect (0, 0, xpos, 20);
  
  // show song title
  fill(255);
  if (currentSong.isPlaying()) {
    String currTime, totalTime, displayText;
    long currMillis, totalMillis;

    currMillis = (long) playPos;
    totalMillis = (long) playLen;
    
    currTime  = String.format("%d:%d", TimeUnit.MILLISECONDS.toMinutes(currMillis),
                                        TimeUnit.MILLISECONDS.toSeconds(currMillis)
                                        - TimeUnit.MINUTES.toSeconds(TimeUnit.MILLISECONDS.toMinutes(currMillis)));
    totalTime = String.format("%d:%d", TimeUnit.MILLISECONDS.toMinutes(totalMillis),
                                        TimeUnit.MILLISECONDS.toSeconds(totalMillis)
                                        - TimeUnit.MINUTES.toSeconds(TimeUnit.MILLISECONDS.toMinutes(totalMillis)));
                                        
    if (currentSong.getMetaData().title().length() > 0) {
      displayText = currentSong.getMetaData().author() + " - " + currentSong.getMetaData().title();
    } else {
      displayText = currentSong.getMetaData().fileName() ;
    }
    text (displayText + " (" + currTime + "/" + totalTime + ")", 2, 0, width, 200);
  } else {
    text ("Pause", 2, 0, width, 200); 
  }


}

// ----- playSound() --------------------------------------------
void playSound() {
  if (isPlaying && !currentSong.isPlaying()) {
    playNextSound();
  }
}

// ----- playNextSound() ----------------------------------------
void playNextSound() {
  if (currentSongIndex >= songList.size() - 1) {
    currentSongIndex = -1;
  }
  currentSong.pause();
  currentSong.rewind();
    
  currentSongIndex++;
  currentSong = songList.get(currentSongIndex);
  
  // init new FFT
  fft = null;
  fft = new FFT(currentSong.bufferSize(), currentSong.sampleRate());
  fft.window(FFT.HAMMING);
  
  currentSong.play();
}

// ----- keyPressed() -------------------------------------------
void keyPressed() {
  if (key == ' ') {
    if (currentSong.isPlaying()) {
      currentSong.pause();
      isPlaying = false;
    } else {
      currentSong.play();
      isPlaying = true;
    }  
  } else if (key == 'n') {
    playNextSound();
  }
}

// ----- mousePressed() -----------------------------------------
void mousePressed() {
  // get click coordinates
  float pos = ((float) mouseX / width) * currentSong.length();
  // jump to position
  currentSong.cue((int) pos);
}

// ----- stop() -------------------------------------------------
void stop()
{
  minim.stop();
  super.stop();
}


